local State = FlexBar2.OO.Mixin
{
	"State_Update",
	"State_Usable",
	"State_Item",
	"SetStateActive",
}

function State:Activate()
	self.State = {};
	self:RegisterEvent("FB2_UPDATE", "State_Update");
	self:RegisterEvent("UNIT_MANA", "State_Usable");
	self:RegisterEvent("UNIT_RAGE", "State_Usable");
	self:RegisterEvent("UNIT_ENERGY", "State_Usable");
	self:RegisterEvent("BAG_UPDATE", "State_Item");
end

function State:SetStateActive(State, StateType)
	if(State == true) then
		if(not FlexBar2:ValueIsInTable(self.State, StateType)) then
			table.insert(self.State, StateType);
			table.sort(self.State, function(a, b)
				return FlexBar2:GetKeyByValue(FlexBar2.db.profile.StatePriority, a) < FlexBar2:GetKeyByValue(FlexBar2.db.profile.StatePriority, b);
			end);
		end
	elseif(State == false) then
		if(FlexBar2:RemoveByValue(self.State, StateType)) then
		end
	end
	if(type(self.State[1]) == "string") then
		local _, r, g, b = FlexBar2.PaintChips:GetRGBPercent(self.State[1])
		self.Frame.Icon:SetVertexColor(r,g,b);
	else
		self.Frame.Icon:SetVertexColor(1,1,1);
	end
end

function State:State_Usable()
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action") then
		if(IsUsableAction(self.Settings.Attributes.action) == 1) then
			self:SetStateActive(false,"OutOfMana")
		else
			self:SetStateActive(true,"OutOfMana")
		end
	else
			self:SetStateActive(false,"OutOfMana")
	end
end

function State:State_Update()
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action") then
		if(ActionHasRange(self.Settings.Attributes.action)) then
			if(not UnitExists("target") or IsActionInRange(self.Settings.Attributes.action) ~= 0) then
				self:SetStateActive(false,"NotInRange")
			else
				self:SetStateActive(true,"NotInRange")
			end
		end
		if(FlexBar2.db.profile.Ticker == "global") then
			local start, duration, enable = GetActionCooldown(self.Settings.Attributes.action);
			if (duration <= 1.5) then
				self:SetStateActive(false,"InCooldown")
			else
				self:SetStateActive(true,"InCooldown")
				self.State.Cooldown = false;
			end
		else
			self:SetStateActive(false,"InCooldown")
		end
	else
		self:SetStateActive(false,"NotInRange")
		self:SetStateActive(false,"InCooldown")
	end
end

function State:State_Item()
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action"
	and IsConsumableAction(self.Settings.Attributes.action) and GetActionCount(self.Settings.Attributes.action) == 0) then
		self:SetStateActive(true,"UnusableItem")
	else
		self:SetStateActive(false,"UnusableItem")
	end
end

FlexBar2:RegisterHandler(State);
